//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MainFrm.h"
#include "FormStatus.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
#include "DLLLeitura.h"
//---------------------------------------------------------------------------
// Handle da DLL de Leitura
HINSTANCE hDLLLeitura;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
// Funo do tipo callback chamada pela DLL de leitura para sinalizar a
// ocorrncia de erros que requerem ao do usurio.
int TrataErrosDLL (long lnErro, char *szDescricao, bool bErroNaoFatal)
{
  // Pede a ao ao usurio
  switch (MessageBox(0, szDescricao, "Erro na Leitura",
                     ((bErroNaoFatal) ? MB_RETRYCANCEL : MB_OK)| MB_ICONERROR))
    {
      case IDRETRY: // Solicitao para retentar a operao
           return DLLErroAcaoRetentativa;

      default: // Deve abortar a leitura
               return DLLErroAcaoCancela;
    }
}
//---------------------------------------------------------------------------
// Alterado em 06/04/2008:
// - Zerado o handle da DLL de leitura <hDLLLeitura> quando DLL  liberada.
void __fastcall TfrmMain::btLeituraClick(TObject *Sender)
{
  // Carrega a DLL de Leitura, se necessrio
  if (!hDLLLeitura)
    hDLLLeitura = LoadLibrary ("LEITURA.DLL");

  // Se DLL no foi encontrada ou no pde ser carregada, apresenta erro
  if (!hDLLLeitura)
    {
      ShowMessage ("Erro na carga da DLL de leitura 'LEITURA.DLL'.");
      return;
    }

  // Obtm o ponteiro para a funo de instalao da funo tratadora
  void __cdecl (*pDLLReadDefineTrataErro) (int (*pTratadora) (long lnErro, char *szDescricao, bool bErroNaoFatal));

  pDLLReadDefineTrataErro = (void __cdecl (*) (int (*pTratadora) (long lnErro, char *szDescricao, bool bErroNaoFatal)))
                            GetProcAddress (hDLLLeitura, "DLLReadDefineTrataErro");


  // Se a funo no existe dentro da DLL, sinaliza erro
  if (!pDLLReadDefineTrataErro)
    {
      // Libera a DLL da memria
      FreeLibrary (hDLLLeitura);

      // Zera o handle da DLL
      hDLLLeitura = 0;

      ShowMessage ("No foi encontrada a funo DLLReadDefineTrataErro dentro da DLL de leitura 'LEITURA.DLL'.");
      return;
    }

  // Instala a funo tratadora de erros de leitura
  (*pDLLReadDefineTrataErro) (TrataErrosDLL);


  // Obtm o ponteiro para a funo de leitura das memrias
  int __cdecl (*pDLLReadLeMemorias) (char *szPortaSerial, char *szNomeArquivo,
                                     char *szSerieECF, char bArguardaConcluirLeitura);

  pDLLReadLeMemorias = (int __cdecl (*) (char *szPortaSerial, char *szNomeArquivo,
                                         char *szSerieECF, char bArguardaConcluirLeitura))
                       GetProcAddress (hDLLLeitura, "DLLReadLeMemorias");


  // Se a funo no existe dentro da DLL, sinaliza erro
  if (!pDLLReadLeMemorias)
    {
      // Libera a DLL da memria
      FreeLibrary (hDLLLeitura);

      // Zera o handle da DLL
      hDLLLeitura = 0;

      ShowMessage ("No foi encontrada a funo DLLReadLeMemorias dentro da DLL de leitura 'LEITURA.DLL'.");
      return;
    }


  // Indica que est processando
  lblCodErro->Caption = "Processando ...";

  // Atualiza o texto da tela ...
  lblCodErro->Update();

  // Dispara a leitura das memrias do ECF
  int Retorno = (*pDLLReadLeMemorias) (cbCanalSerial->Text.c_str(), ebArqBinario->Text.c_str(), "",
                                       (cbAguarda->Checked) ? 1 : 0);

  // Informa o cdigo de erro retornado
  lblCodErro->Caption = AnsiString(Retorno) + " - 0x" + AnsiString(Retorno).IntToHex(Retorno, 4);

  // DLL de leitura pode ser liberada, se no estiver mais em execuo
  if (cbAguarda->Checked)
    {
      FreeLibrary (hDLLLeitura);

      // Zera o handle da DLL
      hDLLLeitura = 0;
    }
   else
    {
      // Seno abre tela de monitorao.
      frmStatus->Show ();
    }
}
//---------------------------------------------------------------------------





void __fastcall TfrmMain::cbTipoPeriodoChange(TObject *Sender)
{
  if (cbTipoPeriodo->Text == "C")
    {
      ebUsuario->Enabled = true;
      lblUsuario->Enabled = true;
    }
   else
    {
      ebUsuario->Enabled = false;
      lblUsuario->Enabled = false;
    }
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btGeraATO17Click(TObject *Sender)
{
  // Carrega a DLL do ATO-17
  HINSTANCE hDLLAto17;
  hDLLAto17 = LoadLibrary ("ATO17.DLL");

  // Se DLL no foi encontrada ou no pde ser carregada, apresenta erro
  if (!hDLLAto17)
    {
      ShowMessage ("Erro na carga da DLL de gerao do ATO-17 'ATO17.DLL'.");
      return;
    }

  // Obtm o ponteiro para a funo de gerao do ATO-17
  int __cdecl (*pDLLATO17GeraArquivo) (char *szArquivoBinario, char *szArquivoTexto,
                                       char *szPeriodoIni, char *szPeriodoFIM,
                                       char TipoPeriodo, char *szUsuario,
                                       char *szTipoLeitura);

  pDLLATO17GeraArquivo = (int __cdecl (*) (char *szArquivoBinario, char *szArquivoTexto,
                                           char *szPeriodoIni, char *szPeriodoFIM,
                                           char TipoPeriodo, char *szUsuario,
                                           char *szTipoLeitura))
                         GetProcAddress (hDLLAto17, "DLLATO17GeraArquivo");


  // Se a funo no existe dentro da DLL, sinaliza erro
  if (!pDLLATO17GeraArquivo)
    {
      // Libera a DLL da memria
      FreeLibrary (hDLLAto17);

      ShowMessage ("No foi encontrada a funo DLLATO17GeraArquivo dentro da DLL de gerao do ATO-17 'ATO17.DLL'.");
      return;
    }

  // Instala a funo tratadora de erros de leitura
  int nCodErro = (*pDLLATO17GeraArquivo) (ebArqBinario->Text.c_str(), ebArqTexto->Text.c_str(),
                                          ebPeriodoDe->Text.c_str(), ebPeriodoAte->Text.c_str(),
                                          *(cbTipoPeriodo->Text.c_str()),
                                          ebUsuario->Text.c_str(),
                                          cbTipoLeitura->Text.c_str());

  // Apresenta o resultado da gerao do arquivo...
  if (nCodErro)
    ShowMessage (AnsiString("Erro na gerao do arquivo com cdigo ") + nCodErro + ".");
   else
    ShowMessage ("Arquivo do ATO-17 gerado com sucesso.");

  // Libera a DLL da memria
  FreeLibrary (hDLLAto17);

}
//---------------------------------------------------------------------------

